@@ -6,4 +6,4 @@ ${MSWINDOWS_DIR}/BUILD.py --light
 echo
 echo "****************************************************************"
 echo
-${MSWINDOWS_DIR}/BUILD.py --no-light
+${MSWINDOWS_DIR}/BUILD.py --no-light --x11
