@@ -37,6 +37,7 @@ PACKAGE_PREFIX = os.environ.get("MINGW_PACKAGE_PREFIX", f"mingw-w64-{MSYSTEM_CAR
 MSYS2_PACKAGE_PREFIX = "msys2-"
 MSYS_DLL_PREFIX = "msys-"
 
+X11_DLL_DIR = os.environ.get("X11_DLL_DIR", "E:/vcxsrv")
 
 TIMESTAMP_SERVER = "http://timestamp.digicert.com"
 # alternative:
@@ -89,6 +90,7 @@ def parse_command_line(argv: list[str]):
     add("service", help="build the system service", default=ARCH != "aarch64")
     add("docs", help="generate the documentation", default=ARCH != "aarch64")
     add("html5", help="bundle the `xpra-html5` client")
+    add("x11", help="include X11 bindings", default=False)
     add("manual", help="bundle the user manual")
     add("numpy", help="bundle `numpy`")
     add("putty", help="bundle putty `plink`")
@@ -153,6 +155,8 @@ def get_build_args(args) -> list[str]:
         xpra_args.append("--with-qt6_client")
     if not args.cuda:
         xpra_args.append("--without-nvidia")
+    if args.x11:
+        xpra_args.append("--with-x11")
     if args.build_args:
         xpra_args += shlex.split(args.build_args)
     # we can't do 'docs' this way :(
@@ -632,7 +636,7 @@ def delete_dlls(light: bool) -> None:
             # no dbus:
             "libdbus*",
             # no AV1:
-            "libaom*", "rav1e*", "libdav1d*", "libheif*",
+            "rav1e*", "libheif*",
             # no avif:
             "libavif*", "libSvt*",
             # remove h264 encoder:
@@ -880,7 +884,9 @@ def bundle_dlls(*expr: str) -> None:
             continue
         for match in matches:
             name = os.path.basename(match)
-            copyfile(match, f"{DIST}/{name}")
+            target = f"{DIST}/{name}"
+            if not os.path.exists(target):
+                copyfile(match, target)
 
 
 def bundle_openssh() -> None:
@@ -923,6 +929,10 @@ def bundle_desktop_logon() -> None:
     bundle_dlls(*dl_dlls)
 
 
+def bundle_x11_dlls() -> None:
+    bundle_dlls(f"{X11_DLL_DIR}/*")
+
+
 def add_cuda(enabled: bool) -> None:
     step(f"cuda: {enabled}")
     if not enabled:
@@ -1069,6 +1079,19 @@ def find_glob_paths(dirname: str, glob_str: str) -> list[str]:
     return output.splitlines()
 
 
+def shorten_package_name(package: str) -> str:
+    if package.startswith(PACKAGE_PREFIX):
+        return package[len(PACKAGE_PREFIX):]
+    if os.path.basename(package).startswith(MSYS_DLL_PREFIX):
+        # ie: "msys-com_err-1.dll"
+        return package
+    if package.startswith(MSYS2_PACKAGE_PREFIX):
+        # ie: "msys2-runtime"
+        return package
+    debug(f"unexpected package prefix: {package!r}")
+    return package
+
+
 def rec_sbom() -> None:
     step("Recording SBOM")
     sbom: dict[str, dict[str, Any]] = {}
@@ -1164,20 +1187,9 @@ def rec_sbom() -> None:
     # summary: list of packages
     packages = tuple(sorted(set(rec["package"] for rec in sbom.values())))
     debug(f"adding package info for {packages}")
-    packages_info: dict[str, tuple] = {}
+    packages_info: dict[str, dict] = {}
     for package_name in packages:
-        # shorten the package name:
-        package = package_name
-        if package.startswith(PACKAGE_PREFIX):
-            package = package[len(PACKAGE_PREFIX):]
-        elif os.path.basename(package).startswith(MSYS_DLL_PREFIX):
-            # ie: "msys-com_err-1.dll"
-            pass  # keep it as it is
-        elif package.startswith(MSYS2_PACKAGE_PREFIX):
-            # ie: "msys2-runtime"
-            pass  # keep it as it is
-        else:
-            debug(f"unexpected package prefix: {package!r}")
+        package = shorten_package_name(package_name)
         # keep only the keys relevant to the sbom:
         info = get_pacman_package_info(package_name) or get_pip_package_info(package_name)
         exported_info = {}
@@ -1395,6 +1407,8 @@ def build(args) -> None:
         bundle_paxec()
     if args.desktop_logon:
         bundle_desktop_logon()
+    if args.x11:
+        bundle_x11_dlls()
     rec_options(args)
     if args.sbom:
         rec_sbom()
