@@ -1,34 +1,40 @@
 #!/usr/bin/env python3
 
-import gi
-gi.require_version('Gtk', '3.0')  # @UndefinedVariable
-gi.require_version('Gdk', '3.0')  # @UndefinedVariable
-from gi.repository import Gtk, Gdk, GLib, GdkX11  # @UnresolvedImport
-assert GdkX11   #this import has side-effects!
+from xpra.os_util import gi_import
 
 from xpra.x11.gtk.display_source import init_gdk_display_source
-init_gdk_display_source()
-from xpra.x11.gtk.prop import prop_get
-from xpra.x11.bindings.window import constants, X11WindowBindings  #@UnresolvedImport
-from xpra.gtk.error import xsync
-X11Window = X11WindowBindings()
+from xpra.x11.prop import prop_get
+from xpra.x11.bindings.core import constants
+from xpra.x11.bindings.window import X11WindowBindings
+from xpra.x11.error import xsync
+
 
-SubstructureNotifyMask = constants["SubstructureNotifyMask"]
-SubstructureRedirectMask = constants["SubstructureRedirectMask"]
 CurrentTime = 0
-root = Gdk.get_default_root_window()
 
 
 def main():
+    init_gdk_display_source()
+
+    gi_import("GdkX11")
+    Gtk = gi_import("Gtk")
+    Gdk = gi_import("Gdk")
+    GLib = gi_import("GLib")
+
+    root = Gdk.get_default_root_window()
+
     win = Gtk.Window()
     win.realize()
+    X11Window = X11WindowBindings()
+
     def print_extents():
         v = prop_get(win.get_window(), "_NET_FRAME_EXTENTS", ["u32"], ignore_errors=False)
         print("_NET_FRAME_EXTENTS: %s" % str(v))
         with xsync:
+            SubstructureNotifyMask = constants["SubstructureNotifyMask"]
+            SubstructureRedirectMask = constants["SubstructureRedirectMask"]
             event_mask = SubstructureNotifyMask | SubstructureRedirectMask
             X11Window.sendClientMessage(root.get_xid(), win.get_window().get_xid(), False, event_mask,
-                      "_NET_REQUEST_FRAME_EXTENTS")
+                                        "_NET_REQUEST_FRAME_EXTENTS")
             print("sending _NET_REQUEST_FRAME_EXTENTS to %#x for %#x" % (root.get_xid(), win.get_window().get_xid()))
         return v is None
     print_extents()
