@@ -1,5 +1,5 @@
 # This file is part of Xpra.
-# Copyright (C) 2017-2024 Antoine Martin <antoine@xpra.org>
+# Copyright (C) 2017 Antoine Martin <antoine@xpra.org>
 # Xpra is released under the terms of the GNU GPL v2, or, at your option, any
 # later version. See the file COPYING for details.
 
@@ -12,14 +12,14 @@ from xpra.common import SocketState
 from xpra.platform.paths import get_xpra_command, get_nodock_command
 from xpra.platform.dotxpra import DotXpra
 from xpra.platform.gui import force_focus
-from xpra.util.child_reaper import getChildReaper
+from xpra.util.child_reaper import get_child_reaper
 from xpra.exit_codes import exit_str
 from xpra.scripts.config import OPTION_TYPES
 from xpra.scripts.main import get_command_args
 from xpra.gtk.window import add_close_accel
 from xpra.gtk.widget import scaled_image, imagebutton, label, modify_fg, color_parse
 from xpra.gtk.pixbuf import get_icon_pixbuf
-from xpra.gtk.signals import register_os_signals
+from xpra.util.glib import register_os_signals
 from xpra.gtk.dialogs.util import hb_button
 from xpra.net.common import DEFAULT_PORTS
 from xpra.exit_codes import ExitCode, ExitValue
@@ -148,7 +148,6 @@ class SessionsGUI(Gtk.Window):
 
         self.clients = {}
         self.clients_disconnecting = set()
-        self.child_reaper = getChildReaper()
 
         self.vbox = Gtk.VBox(homogeneous=False, spacing=20)
         self.add(self.vbox)
@@ -421,7 +420,7 @@ class SessionsGUI(Gtk.Window):
 
                 GLib.idle_add(update)
 
-        self.child_reaper.add_process(proc, "client-%s" % uri, cmd, True, True, proc_exit)
+        get_child_reaper().add_process(proc, "client-%s" % uri, cmd, True, True, proc_exit)
         self.clients[key] = proc
         self.populate()
 
