@@ -7,11 +7,23 @@
 import os
 from collections.abc import Callable, Sequence
 
-from xpra.os_util import gi_import
 from xpra.platform.keyboard_base import KeyboardBase, log
-from xpra.platform.darwin.menu import getOSXMenuHelper
+from xpra.util.str_fn import Ellipsizer
 
-Gdk = gi_import("Gdk")
+# from xpra.os_util import gi_import
+# Gdk = gi_import("Gdk")
+# Gdk.ModifierType.SHIFT_MASK: 1
+# Gdk.ModifierType.LOCK_MASK: 2
+# Gdk.ModifierType.SUPER_MASK: 67108864,
+# Gdk.ModifierType.HYPER_MASK: 134217728,
+# Gdk.ModifierType.META_MASK: 268435456,
+# Gdk.ModifierType.CONTROL_MASK: 4,
+SHIFT_MASK = 1
+LOCK_MASK = 2
+SUPER_MASK = 67108864
+HYPER_MASK = 134217728
+META_MASK = 268435456
+CONTROL_MASK = 4
 
 NUM_LOCK_KEYCODE = 71  # HARDCODED!
 # a key and the keys we want to translate it into when swapping keys
@@ -70,6 +82,50 @@ APPLE_LAYOUTS: dict[str, str] = {
 }
 
 
+def fake_key(keycode: int, press: bool) -> None:
+    import Quartz.CoreGraphics as CG
+    e = CG.CGEventCreateKeyboardEvent(None, keycode, press)
+    log("fake_key(%s, %s)", keycode, press)
+    # CGEventSetFlags(keyPress, modifierFlags)
+    # modifierFlags: kCGEventFlagMaskShift, ...
+    CG.CGEventPost(CG.kCGSessionEventTap, e)
+    # this causes crashes, don't do it!
+    # CG.CFRelease(e)
+
+
+class CGFakeKeyboard:
+
+    @staticmethod
+    def press_key(keycode: int, press: bool) -> None:
+        fake_key(keycode, press)
+
+    @staticmethod
+    def clear_keys_pressed(_keycodes) -> None:
+        pass
+
+    @staticmethod
+    def set_repeat_rate(self, delay: int, interval: int) -> None:
+        """
+        This can't be done?
+        The closest I can find is:
+        `defaults write NSGlobalDomain KeyRepeat -int <rate_value>`
+        `defaults write NSGlobalDomain InitialKeyRepeat -int <delay_value>`
+        But this requires the user to logout and then log back in again!
+        """
+
+    @staticmethod
+    def get_keycodes_down() -> Sequence[int]:
+        return ()
+
+    @staticmethod
+    def get_layout_group() -> int:
+        return 0
+
+
+def get_keyboard_device():
+    return CGFakeKeyboard()
+
+
 class Keyboard(KeyboardBase):
     """
         Switch Meta and Control
@@ -112,7 +168,7 @@ class Keyboard(KeyboardBase):
             code = APPLE_LAYOUTS.get(current_keyboard.split(".")[-1])
             log("get_layout_spec() current_keyboard=%s, code=%s", current_keyboard, code)
             all_keyboards = text_input_context.keyboardInputSources()
-            log("get_layout_spec() other keyboards=%s", repr(all_keyboards))
+            log("get_layout_spec() other keyboards=%s", Ellipsizer(all_keyboards))
             if code:
                 layout = code
             if all_keyboards:
@@ -126,8 +182,8 @@ class Keyboard(KeyboardBase):
             else:
                 if code and code not in layouts:
                     layouts.insert(0, code)
-            log("get_layout_spec() view=%s, input_context=%s, layout=%s, layouts=%s",
-                repr(view), text_input_context, layout, layouts)
+            log("get_layout_spec() view=%r, input_context=%s", view, text_input_context)
+            log("get_layout_spec() layout=%s, layouts=%s", layout, layouts)
         except Exception as e:
             log("get_layout_spec()", exc_info=True)
             log.error("Error querying keyboard layout:")
@@ -205,12 +261,12 @@ class Keyboard(KeyboardBase):
             meta = self.meta_modifier
             control = self.control_modifier
         modmap = {
-            Gdk.ModifierType.SHIFT_MASK: "shift",
-            Gdk.ModifierType.LOCK_MASK: "lock",
-            Gdk.ModifierType.SUPER_MASK: self.super_modifier,
-            Gdk.ModifierType.HYPER_MASK: self.hyper_modifier,
-            Gdk.ModifierType.META_MASK: meta,
-            Gdk.ModifierType.CONTROL_MASK: control,
+            SHIFT_MASK: "shift",
+            LOCK_MASK: "lock",
+            SUPER_MASK: self.super_modifier,
+            HYPER_MASK: self.hyper_modifier,
+            META_MASK: meta,
+            CONTROL_MASK: control,
         }
         names = []
         for modmask, modname in modmap.items():
@@ -236,7 +292,12 @@ class Keyboard(KeyboardBase):
             if not key_event.pressed:
                 log("toggling numlock")
                 self.num_lock_state = not self.num_lock_state
-                getOSXMenuHelper().update_numlock(self.num_lock_state)
+                try:
+                    from xpra.platform.darwin.menu import getOSXMenuHelper
+                except ImportError:
+                    pass
+                else:
+                    getOSXMenuHelper().update_numlock(self.num_lock_state)
             # do not forward the "Escape" key that numlock usually comes up as
             return
         send_key_action_cb(wid, key_event)
