@@ -20,7 +20,7 @@ def debug(*msg):
 _gtkosx_warning_ = False
 
 
-def do_get_resources_dir():
+def do_get_resources_dir() -> str:
     rsc = None
     RESOURCES = "/Resources/"
     # FUGLY warning: importing gtkosx_application causes the dock to appear,
@@ -33,7 +33,7 @@ def do_get_resources_dir():
             debug("get_resources_dir() %s.get_resource_path=%s", macapp, rsc)
         except Exception:
             global _gtkosx_warning_
-            if _gtkosx_warning_ is False:
+            if not _gtkosx_warning_:
                 _gtkosx_warning_ = True
                 # delayed import to prevent cycles:
                 get_util_logger().error("Error: gtkosx_application module is missing - trying to continue anyway")
@@ -231,7 +231,7 @@ def do_get_nodock_command() -> list[str]:
     # AppStore builds have script wrappers:
     helper = os.path.join(base, "Resources", "scripts", "Xpra")
     if not os.path.exists(helper):
-        helper = os.path.join(_get_helpers_dir(), "Xpra")
+        helper = os.path.join(base, "Helpers", "Xpra")
     if not os.path.exists(helper):
         # having a dock is still better than
         # trying to run a command that does not exist!
