@@ -1,5 +1,5 @@
 # This file is part of Xpra.
-# Copyright (C) 2013-2024 Antoine Martin <antoine@xpra.org>
+# Copyright (C) 2013 Antoine Martin <antoine@xpra.org>
 # Xpra is released under the terms of the GNU GPL v2, or, at your option, any
 # later version. See the file COPYING for details.
 
@@ -15,9 +15,9 @@ from xpra.scripts.config import InitExit
 from xpra.scripts.main import check_display
 from xpra.exit_codes import ExitCode
 from xpra.codecs.image import ImageWrapper
-from xpra.server.gtk_server import GTKServerBase
 from xpra.server.shadow.gtk_shadow_server_base import GTKShadowServerBase
 from xpra.platform.darwin.keyboard_config import KeyboardConfig
+from xpra.platform.darwin.pointer import click, move_pointer
 from xpra.platform.darwin.gui import get_CG_imagewrapper, take_screenshot
 from xpra.log import Logger
 
@@ -76,25 +76,30 @@ class OSXRootCapture:
     def __repr__(self):
         return "OSXRootCapture"
 
-    def get_type(self) -> str:
+    @staticmethod
+    def get_type() -> str:
         return "CoreGraphics"
 
-    def refresh(self) -> bool:
+    @staticmethod
+    def refresh() -> bool:
         return True
 
     def clean(self) -> None:
         """ nothing specific to do here on MacOS """
 
-    def get_image(self, x, y, width, height) -> ImageWrapper:
+    @staticmethod
+    def get_image(x, y, width, height) -> ImageWrapper:
         rect = (x, y, width, height)
         return get_CG_imagewrapper(rect)
 
-    def get_info(self) -> dict[str, Any]:
+    @staticmethod
+    def get_info() -> dict[str, Any]:
         return {
             "type": "CoreGraphics",
         }
 
-    def take_screenshot(self) -> tuple[int, int, str, int, bytes]:
+    @staticmethod
+    def take_screenshot() -> tuple[int, int, str, int, bytes]:
         log("grabbing screenshot")
         return take_screenshot()
 
@@ -124,11 +129,12 @@ class ShadowServer(GTKShadowServerBase):
         # printing fails silently on OSX
         self.printing = False
 
-    def get_keyboard_config(self, _props=None) -> KeyboardConfig:
+    @staticmethod
+    def get_keyboard_config(_props=None) -> KeyboardConfig:
         return KeyboardConfig()
 
     def make_tray_widget(self):
-        from xpra.client.gtk3.statusicon_tray import GTKStatusIconTray
+        from xpra.gtk.statusicon_tray import GTKStatusIconTray
         return GTKStatusIconTray(self, 0, self.tray, "Xpra Shadow Server",
                                  click_cb=self.tray_click_callback, mouseover_cb=None, exit_cb=self.tray_exit_callback)
 
@@ -198,18 +204,10 @@ class ShadowServer(GTKShadowServerBase):
         if proto not in self._server_sources:
             return False
         assert wid in self._id_to_window
-        CG.CGWarpMouseCursorPosition(pointer[:2])
+        x, y = pointer[:2]
+        move_pointer(x, y)
         return True
 
-    def fake_key(self, keycode: int, press: bool) -> None:
-        e = CG.CGEventCreateKeyboardEvent(None, keycode, press)
-        log("fake_key(%s, %s)", keycode, press)
-        # CGEventSetFlags(keyPress, modifierFlags)
-        # modifierFlags: kCGEventFlagMaskShift, ...
-        CG.CGEventPost(CG.kCGSessionEventTap, e)
-        # this causes crashes, don't do it!
-        # CG.CFRelease(e)
-
     def do_process_button_action(self, proto, device_id: int, wid: int, button: int, pressed: bool, pointer, props):
         if "modifiers" in props:
             self._update_modifiers(proto, wid, props.get("modifiers"))
@@ -217,39 +215,22 @@ class ShadowServer(GTKShadowServerBase):
         if pointer:
             self.button_action(device_id, wid, pointer, button, pressed, props)
 
+    def _move_pointer(self, device_id: int, wid: int, pos, props=None) -> None:
+        x, y = pos[:2]
+        move_pointer(x, y)
+
     def button_action(self, device_id: int, wid: int, pointer, button: int, pressed: bool, props):
-        if button <= 3:
-            # we should be using CGEventCreateMouseEvent
-            # instead we clear previous clicks when a "higher" button is pressed... oh well
-            event = [pointer[:2], 1, button]
-            for i in range(button):
-                event.append(i == (button - 1) and pressed)
-            r = CG.CGPostMouseEvent(*event)
-            log("CG.CGPostMouseEvent%s=%s", event, r)
-            return
-        if not pressed:
-            # we don't simulate press/unpress
-            # so just ignore unpressed events
-            return
-        wheel = (button - 2) // 2
-        direction = 1 - (((button - 2) % 2) * 2)
-        event = [wheel]
-        for i in range(wheel):
-            if i != (wheel - 1):
-                event.append(0)
-            else:
-                event.append(direction)
-        r = CG.CGPostScrollWheelEvent(*event)
-        log("CG.CGPostScrollWheelEvent%s=%s", event, r)
+        x, y = pointer[:2]
+        click(x, y, button, pressed)
 
     def make_hello(self, source) -> dict[str, Any]:
-        capabilities = GTKServerBase.make_hello(self, source)
+        capabilities = super().make_hello(source)
         capabilities["shadow"] = True
         capabilities["server_type"] = "Python/MacOS-Shadow"
         return capabilities
 
-    def get_info(self, proto, *_args) -> dict[str, Any]:
-        info = GTKServerBase.get_info(self, proto)
+    def get_threaded_info(self, proto, **kwargs) -> dict[str, Any]:
+        info = super().get_threaded_info(proto, **kwargs)
         info.setdefault("features", {})["shadow"] = True
         info.setdefault("server", {})["type"] = "Python/gtk2/osx-shadow"
         info.setdefault("damage", {}).update({
@@ -261,7 +242,7 @@ class ShadowServer(GTKShadowServerBase):
         return info
 
 
-def main():
+def main() -> None:
     import sys
     from xpra.platform import program_context
     with program_context("MacOS Shadow Capture"):
