@@ -11,18 +11,25 @@ import sys
 import site
 import tempfile
 
+from collections.abc import Sequence
 
-def do_get_desktop_background_paths():
+from xpra.common import BACKWARDS_COMPATIBLE
+
+
+def do_get_desktop_background_paths() -> Sequence[str]:
     return [
         "/usr/share/backgrounds/images/default.png",
         "/usr/share/backgrounds/images/*default*.png",
         "/usr/share/backgrounds/*default*png",
         "/usr/share/backgrounds/gnome/adwaita*.jpg",  # Debian Stretch
         "/usr/share/backgrounds/images/*jpg",  # CentOS 7
+        "/usr/share/backgrounds/images/default.jxl",  # Fedora 42
+        "/usr/share/backgrounds/images/*default*.jxl",
+        "/usr/share/backgrounds/images/*.jxl",
     ]
 
 
-def do_get_install_prefix():
+def do_get_install_prefix() -> str:
     # special case for "user" installations, ie:
     # $HOME/.local/lib/python3.8/site-packages/xpra/platform/paths.py
     try:
@@ -38,7 +45,7 @@ def do_get_install_prefix():
     return sys.prefix
 
 
-def do_get_resources_dir():
+def do_get_resources_dir() -> str:
     # is there a better/cleaner way?
     from xpra.common import DEFAULT_XDG_DATA_DIRS
     from xpra.platform.paths import get_install_prefix
@@ -60,32 +67,31 @@ def do_get_resources_dir():
     return os.getcwd()
 
 
-def do_get_app_dir():
+def do_get_app_dir() -> str:
     from xpra.platform.paths import get_resources_dir
     return get_resources_dir()
 
 
-def do_get_image_dir():
+def do_get_image_dir() -> str:
     from xpra.platform.paths import get_app_dir
     return os.path.join(get_app_dir(), "images")
 
 
-def do_get_icon_dir():
+def do_get_icon_dir() -> str:
     from xpra.platform.paths import get_app_dir
     return os.path.join(get_app_dir(), "icons")
 
 
-def do_get_mmap_dir():
+def do_get_mmap_dir() -> str:
     return _get_xpra_runtime_dir() or tempfile.gettempdir()
 
 
-def do_get_xpra_tmp_dir():
+def do_get_xpra_tmp_dir() -> str:
     xrd = os.environ.get("XPRA_SESSION_DIR", _get_xpra_runtime_dir())
     return os.path.join(xrd, "tmp")
 
 
-def do_get_script_bin_dirs():
-    # versions before 0.17 only had "~/.xpra/run-xpra"
+def do_get_script_bin_dirs() -> list[str]:
     script_bin_dirs = []
     runtime_dir = _get_xpra_runtime_dir()
     if runtime_dir:
@@ -93,7 +99,7 @@ def do_get_script_bin_dirs():
     return script_bin_dirs
 
 
-def do_get_system_conf_dirs():
+def do_get_system_conf_dirs() -> list[str]:
     dirs = ["/etc/xpra", "/usr/local/etc/xpra"]
     for d in os.environ.get("XDG_CONFIG_DIRS", "/etc/xdg").split(":"):
         dirs.append(os.path.join(d, "xpra"))
@@ -121,7 +127,7 @@ def do_get_system_menu_dirs() -> list[str]:
     return menu_dirs
 
 
-def do_get_state_dir():
+def do_get_state_dir() -> str:
     if os.getuid() > 0:
         xdg_state = os.path.expanduser(os.environ.get("XDG_STATE_HOME", "~/.local/state"))
         state_dir = os.path.expanduser(xdg_state)
@@ -133,7 +139,7 @@ def do_get_state_dir():
     return "~/.xpra/state"
 
 
-def do_get_user_conf_dirs(uid):
+def do_get_user_conf_dirs(uid) -> list[str]:
     # per-user configuration location:
     # (but never use /root/.xpra or /root/.config/xpra)
     if uid is None:
@@ -145,8 +151,8 @@ def do_get_user_conf_dirs(uid):
     return dirs
 
 
-def get_runtime_dir():
-    runtime_dir = os.environ.get("XDG_RUNTIME_DIR")
+def get_runtime_dir() -> str:
+    runtime_dir = os.environ.get("XDG_RUNTIME_DIR", "")
     if runtime_dir:
         return runtime_dir
     if sys.platform.startswith("linux"):
@@ -159,32 +165,34 @@ def get_runtime_dir():
     return runtime_dir
 
 
-def _get_xpra_runtime_dir():
+def _get_xpra_runtime_dir() -> str:
     runtime_dir = get_runtime_dir()
     if not runtime_dir:
-        return None
+        return ""
     return os.path.join(runtime_dir, "xpra")
 
 
-def do_get_socket_dirs():
+def do_get_socket_dirs() -> list[str]:
     SOCKET_DIRS = []
-    runtime_dir = _get_xpra_runtime_dir()
-    if runtime_dir:
-        # private, per user: XDG_RUNTIME_DIR/xpra
-        # (ie: "/run/user/1000/xpra")
-        SOCKET_DIRS.append(runtime_dir)
+    if BACKWARDS_COMPATIBLE:
+        runtime_dir = _get_xpra_runtime_dir()
+        if runtime_dir:
+            # private, per user: XDG_RUNTIME_DIR/xpra
+            # (ie: "/run/user/1000/xpra")
+            SOCKET_DIRS.append(runtime_dir)
     # for shared sockets (the 'xpra' group should own this directory):
     if os.path.exists("/run"):
         SOCKET_DIRS.append("/run/xpra")
     elif os.path.exists("/var/run"):
         SOCKET_DIRS.append("/var/run/xpra")
-    # Debian and Ubuntu often don't create a reliable XDG_RUNTIME_DIR
-    # other distros may not create one when using "su"
-    SOCKET_DIRS.append("~/.xpra")
+    if BACKWARDS_COMPATIBLE:
+        # Debian and Ubuntu often don't create a reliable XDG_RUNTIME_DIR
+        # other distros may not create one when using "su"
+        SOCKET_DIRS.append("~/.xpra")
     return SOCKET_DIRS
 
 
-def do_get_client_socket_dirs():
+def do_get_client_socket_dirs() -> list[str]:
     DIRS = []
     runtime_dir = _get_xpra_runtime_dir()
     if runtime_dir:
@@ -192,7 +200,7 @@ def do_get_client_socket_dirs():
     return DIRS
 
 
-def do_get_default_log_dirs():
+def do_get_default_log_dirs() -> list[str]:
     log_dirs = []
     v = _get_xpra_runtime_dir()
     if v:
@@ -201,12 +209,12 @@ def do_get_default_log_dirs():
     return log_dirs
 
 
-def do_get_audio_command():
+def do_get_audio_command() -> list[str]:
     from xpra.platform.paths import get_xpra_command
     return get_xpra_command()
 
 
-def do_get_xpra_command():
+def do_get_xpra_command() -> list[str]:
     # try to use the same "xpra" executable that launched this server,
     # whilst also preserving the python interpreter version:
     if sys.argv and sys.argv[0].lower().endswith("/xpra"):
