@@ -1,6 +1,6 @@
 #!/usr/bin/env python3
 # This file is part of Xpra.
-# Copyright (C) 2016-2024 Antoine Martin <antoine@xpra.org>
+# Copyright (C) 2016 Antoine Martin <antoine@xpra.org>
 # Xpra is released under the terms of the GNU GPL v2, or, at your option, any
 # later version. See the file COPYING for details.
 
@@ -8,7 +8,7 @@ from collections.abc import Callable
 
 from xpra.os_util import gi_import
 from xpra.platform import platform_import
-from xpra.log import Logger
+from xpra.log import Logger, consume_verbose_argv
 
 log = Logger("webcam")
 
@@ -17,7 +17,7 @@ def get_virtual_video_devices() -> dict:
     return {}
 
 
-def get_all_video_devices():
+def get_all_video_devices() -> dict | None:
     # None means we can't enumerate,
     # this is different from an empty dict!
     return None
@@ -37,11 +37,11 @@ _video_device_change_callbacks: list[Callable] = []
 
 
 def _fire_video_device_change(create=None, pathname=None) -> None:
-    for x in _video_device_change_callbacks:
+    for callback in _video_device_change_callbacks:
         try:
-            x(create, pathname)
+            callback(create, pathname)
         except Exception as e:
-            log("error on %s", x, exc_info=True)
+            log("error on %s", callback, exc_info=True)
             log.error("Error: video device change callback error")
             log.estr(e)
 
@@ -54,15 +54,11 @@ platform_import(globals(), "webcam", False,
 
 
 def main(argv) -> int:
-    # pylint: disable=import-outside-toplevel
-    if "-v" in argv or "--verbose" in argv:
-        from xpra.log import add_debug_category, enable_debug_for
-        enable_debug_for("webcam")
-        add_debug_category("webcam")
     run = "-r" in argv or "--run" in argv
     from xpra.util.str_fn import print_nested_dict
     from xpra.platform import program_context
     with program_context("Webcam Info", "Webcam Info"):
+        consume_verbose_argv(argv, "webcam")
         devices = get_virtual_video_devices() or {}
         log.info("Found %i virtual video devices:", len(devices))
         print_nested_dict(devices)
