@@ -118,6 +118,7 @@ class ProxyServer(_ProxyServer):
             # "--tray=no",
         ]
         # unless explicitly stated otherwise, exit with client:
+        # noinspection PySimplifyBooleanCheck
         if sess_options.get("exit-with-client", None) is not False:
             cmd.append("--exit-with-client=yes")
         from xpra.log import debug_enabled_categories
@@ -155,5 +156,6 @@ class ProxyServer(_ProxyServer):
                     # TODO: test the named pipe
                     sleep(2)
                     break
-        self.child_reaper.add_process(proc, f"server-{username}", "xpra shadow", True, True)
+        from xpra.util.child_reaper import get_child_reaper
+        get_child_reaper().add_process(proc, f"server-{username}", "xpra shadow", True, True)
         return proc, f"named-pipe://{named_pipe}", named_pipe
