@@ -10,11 +10,11 @@ from time import monotonic
 from typing import Any
 
 from xpra.server.core import ServerCore
-from xpra.server.background_worker import add_work_item
-from xpra.common import FULL_INFO, noop, ConnectionMessage
-from xpra.net.common import PacketType, PacketElement
-from xpra.scripts.config import str_to_bool
+from xpra.util.background_worker import add_work_item
+from xpra.common import FULL_INFO, noop, subsystem_name, ConnectionMessage, BACKWARDS_COMPATIBLE
+from xpra.net.common import Packet, PacketElement
 from xpra.os_util import WIN32, gi_import
+from xpra.platform.dotxpra import DotXpra
 from xpra.util.objects import typedict, merge_dicts
 from xpra.util.str_fn import csv, Ellipsizer
 from xpra.util.env import envbool
@@ -30,8 +30,12 @@ netlog = Logger("network")
 authlog = Logger("auth")
 
 SERVER_BASES = get_server_base_classes()
+SIGNALS: dict[str, tuple] = {}
+for base_class in SERVER_BASES:
+    SIGNALS.update(getattr(base_class, "__signals__", {}))
 ServerBaseClass = type("ServerBaseClass", SERVER_BASES, {})
 log("ServerBaseClass%s", SERVER_BASES)
+log("signals: %s", SIGNALS)
 
 CLIENT_CAN_SHUTDOWN = envbool("XPRA_CLIENT_CAN_SHUTDOWN", True)
 MDNS_CLIENT_COUNT = envbool("XPRA_MDNS_CLIENT_COUNT", True)
@@ -42,14 +46,19 @@ class ServerBase(ServerBaseClass):
     This is the base class for seamless and desktop servers. (not proxy servers)
     It provides all the generic functions but is not tied
     to a specific backend (X11 or otherwise).
-    See GTKServerBase/X11ServerBase and other platform specific subclasses.
+    See X11ServerBase and other platform specific subclasses.
     """
+    __signals__ = SIGNALS
 
     def __init__(self):
         for c in SERVER_BASES:
             c.__init__(self)
         log("ServerBase.__init__()")
-
+        self.hello_request_handlers.update({
+            "detach": self._handle_hello_request_detach,
+            "exit": self._handle_hello_request_exit,
+            "stop": self._handle_hello_request_stop,
+        })
         self._server_sources: dict = {}
         self.client_properties: dict[int, dict] = {}
         self.ui_driver = None
@@ -57,16 +66,16 @@ class ServerBase(ServerBaseClass):
         self.lock: bool | None = None
         self.idle_timeout: int = 0
         self.client_shutdown: bool = CLIENT_CAN_SHUTDOWN
+        self.dotxpra = None
 
-    def suspend_event(self, args) -> None:
-        ServerCore.suspend_event(self, args)
+    def suspend_event(self, *_args) -> None:
+        # if we get a `suspend_event`, we can assume that `PowerEventServer` is a superclass:
         self.server_event("suspend")
         for s in self._server_sources.values():
             if hasattr(s, "go_idle"):
                 s.go_idle()
 
-    def resume_event(self, args) -> None:
-        ServerCore.resume_event(self, args)
+    def resume_event(self, *_args) -> None:
         self.server_event("resume")
         for s in self._server_sources.values():
             if hasattr(s, "no_idle"):
@@ -84,6 +93,7 @@ class ServerBase(ServerBaseClass):
         return self._server_sources.get(proto)
 
     def init(self, opts) -> None:
+        self.dotxpra = DotXpra(opts.socket_dir, opts.socket_dirs + opts.client_socket_dirs)
         # from now on, use the logger for parsing errors:
         from xpra.scripts import config  # pylint: disable=import-outside-toplevel
         config.warn = log.warn
@@ -104,16 +114,15 @@ class ServerBase(ServerBaseClass):
             end = monotonic()
             log("%3ims in %s.setup", 1000 * (end - start), c)
 
-    def threaded_init(self) -> None:
+    def threaded_setup(self) -> None:
         log("threaded_init() serverbase start")
         for c in SERVER_BASES:
             with log.trap_error("Error during threaded setup of %s", c):
                 c.threaded_setup(self)
         log("threaded_init() serverbase end")
-        super().call_init_thread_callbacks()
 
     def server_is_ready(self) -> None:
-        ServerCore.server_is_ready(self)
+        super().server_is_ready()
         self.server_event("ready")
 
     def do_cleanup(self) -> None:
@@ -123,7 +132,10 @@ class ServerBase(ServerBaseClass):
         for c in reversed(SERVER_BASES):
             if c != ServerCore:
                 log("%s", c.cleanup)
-                c.cleanup(self)
+                try:
+                    c.cleanup(self)
+                except Exception:
+                    log.warn(f"Error: in {c} cleanup", exc_info=True)
 
     def late_cleanup(self, stop=True) -> None:
         for c in reversed(SERVER_BASES):
@@ -142,26 +154,40 @@ class ServerBase(ServerBaseClass):
 
     ######################################################################
     # shutdown / exit commands:
-    def _process_exit_server(self, _proto, packet: PacketType = ("exit-server", )) -> None:
-        if not self.client_shutdown:
-            log.warn("Warning: ignoring exit request")
-            return
-        reason: ConnectionMessage | str = ConnectionMessage.SERVER_EXIT
-        message = "Exiting in response to client request"
+    def _process_exit_server(self, _proto, packet: Packet = Packet("exit-server")) -> None:
+        # packet from a client
+        reason = ""
         if len(packet) > 1:
-            reason = str(packet[1])
-            message += ": " + reason
+            reason = packet.get_str(1)
+        self._request_exit(reason)
+
+    def _handle_hello_request_exit(self, _proto, _caps: typedict) -> bool:
+        # handle "xpra exit" hello request
+        self._request_exit()
+        return True
+
+    def _request_exit(self, reason: ConnectionMessage | str = "") -> None:
+        message = "Exiting in response to client request"
+        if reason:
+            message += f": {reason}"
         log.info(message)
         self.cleanup_all_protocols(reason=reason)
         GLib.timeout_add(500, self.clean_quit, ServerExitMode.EXIT)
 
-    def _process_shutdown_server(self, _proto, _packet: PacketType = ("shutdown-server", )) -> None:
+    def _process_shutdown_server(self, _proto, _packet: Packet = Packet("shutdown-server")) -> None:
+        self._request_stop()
+
+    def _handle_hello_request_stop(self, _proto, _caps: typedict) -> bool:
+        return self._request_stop()
+
+    def _request_stop(self) -> bool:
         if not self.client_shutdown:
             log.warn("Warning: ignoring shutdown request")
-            return
+            return False
         log.info("Shutting down in response to client request")
         self.cleanup_all_protocols(reason=ConnectionMessage.SERVER_SHUTDOWN)
         GLib.timeout_add(500, self.clean_quit)
+        return True
 
     def get_mdns_info(self) -> dict[str, Any]:
         mdns_info = ServerCore.get_mdns_info(self)
@@ -171,6 +197,7 @@ class ServerBase(ServerBaseClass):
 
     ######################################################################
     # handle new connections:
+    # noinspection PySimplifyBooleanCheck
     def handle_sharing(self, proto, ui_client: bool = True, share: bool = False,
                        uuid="") -> tuple[bool, int]:
         share_count = 0
@@ -230,7 +257,7 @@ class ServerBase(ServerBaseClass):
             log.warn("Warning: received another 'hello' packet")
             log.warn(" from an existing connection: %s", proto)
             return
-        if ServerCore.hello_oked(self, proto, c, auth_caps):
+        if super().hello_oked(proto, c, auth_caps):
             # has been handled
             return
         if not self.sanity_checks(proto, c):
@@ -268,29 +295,15 @@ class ServerBase(ServerBaseClass):
             ss.close()
             raise
         self._server_sources[proto] = ss
-        add_work_item(self.mdns_update)
         # process ui half in ui thread:
         GLib.idle_add(self.process_hello_ui, ss, c, auth_caps, ui_client, share_count)
 
-    def do_handle_hello_request(self, request: str, proto, caps: typedict) -> bool:
-        if super().do_handle_hello_request(request, proto, caps):
-            return True
-        if request == "detach":
-            self.detach_server(proto)
-            return True
-        if request == "exit":
-            self._process_exit_server(proto)
-            return True
-        if request == "stop":
-            self._process_shutdown_server(proto)
-            return True
-        return False
-
-    def detach_server(self, proto) -> None:
+    def _handle_hello_request_detach(self, proto, _caps: typedict) -> bool:
+        # noinspection PySimplifyBooleanCheck
         if self.lock is True:
             authlog("cannot detach: session is locked")
             self.disconnect_client(proto, ConnectionMessage.SESSION_BUSY, "this session is locked")
-            return
+            return False
         count = locked = 0
         for p, ss in tuple(self._server_sources.items()):
             if p != proto:
@@ -304,11 +317,12 @@ class ServerBase(ServerBaseClass):
         if locked:
             message += f", {locked} still have it locked"
         self.disconnect_client(proto, ConnectionMessage.DONE, message)
+        return True
 
     @staticmethod
     def get_client_connection_class(caps: typedict) -> type:
         # pylint: disable=import-outside-toplevel
-        from xpra.server.source.client_connection_factory import get_client_connection_class
+        from xpra.server.source.factory import get_client_connection_class
         return get_client_connection_class(caps)
 
     def process_hello_ui(self, ss, c: typedict, auth_caps: dict, send_ui: bool, share_count: int) -> None:
@@ -330,6 +344,7 @@ class ServerBase(ServerBaseClass):
 
             self.send_hello(ss, auth_caps)
             self.add_new_client(ss, c, send_ui, share_count)
+            self.update_all_server_settings(share_count == 0)  # if we're not sharing, reset all the settings
             self.send_initial_data(ss, c, send_ui, share_count)
             self.client_startup_complete(ss)
 
@@ -403,8 +418,8 @@ class ServerBase(ServerBaseClass):
                 "sharing-toggle": self.sharing is None,
                 "lock": self.lock is not False,
                 "lock-toggle": self.lock is None,
-                "windows": features.windows,
-                "keyboard": features.input_devices,
+                "windows": features.window,
+                "keyboard": features.keyboard,
             }
             sf = self.get_server_features(source)
             capabilities.update(sf)
@@ -415,81 +430,72 @@ class ServerBase(ServerBaseClass):
         if server_cipher:
             capabilities.update(server_cipher)
         server_source.send_hello(capabilities)
-        self.after_threaded_init(server_source.threaded_init_complete, self)
 
     ######################################################################
     # utility method:
     def window_sources(self) -> tuple:
-        from xpra.server.source.windows import WindowsMixin  # pylint: disable=import-outside-toplevel
-        return tuple(x for x in self._server_sources.values() if isinstance(x, WindowsMixin))
+        from xpra.server.source.window import WindowsConnection  # pylint: disable=import-outside-toplevel
+        return tuple(x for x in self._server_sources.values() if isinstance(x, WindowsConnection))
 
     ######################################################################
     # info:
-    def _process_info_request(self, proto, packet: PacketType) -> None:
-        log("process_info_request(%s, %s)", proto, packet)
-        # ignoring the list of client uuids supplied in packet[1]
-        ss = self.get_server_source(proto)
-        if not ss:
-            return
-
-        try:
-            options = proto._conn.options
-            info_option = options.get("info", "yes")
-        except AttributeError:
-            info_option = "yes"
-        if not str_to_bool(info_option):
-            err = "`info` commands are not enabled on this connection"
-            log.warn(f"Warning: {err}")
-            ss.send_info_response({"error": err})
-            return
-
-        categories: list[str] = []
-        # if len(packet>=2):
-        #    uuid = packet[1]
-        if len(packet) >= 4:
-            categories = [str(x) for x in packet[3]]
-
-        def info_callback(_proto, info) -> None:
-            assert proto == _proto
-            if categories:
-                info = {k: v for k, v in info.items() if k in categories}
-            ss.send_info_response(info)
-
-        self.get_all_info(info_callback, proto, None)
-
-    def send_hello_info(self, proto) -> None:
+    def send_hello_info(self, proto, **kwargs) -> None:
         self.wait_for_threaded_init()
-        start = monotonic()
+        super().send_hello_info(proto, **kwargs)
 
-        def cb(iproto, info) -> None:
-            self.do_send_info(iproto, info)
-            end = monotonic()
-            log.info("processed info request from %s in %ims",
-                     iproto._conn, (end - start) * 1000)
-
-        self.get_all_info(cb, proto, None)
-
-    def get_ui_info(self, proto, client_uuids=None, *args) -> dict[str, Any]:
+    def get_ui_info(self, proto, **kwargs) -> dict[str, Any]:
         """ info that must be collected from the UI thread
             (ie: things that query the display)
         """
         info: dict[str, Any] = {}
+        subsystems = kwargs.get("subsystems", ())
+        log("ServerBase.get_ui_info(%s, %s) subsystems=%s", proto, kwargs, subsystems)
         for c in SERVER_BASES:
+            if subsystems and subsystem_name(c) not in subsystems:
+                continue
             with log.trap_error("Error collecting UI info from %s", c):
-                mixin_info = c.get_ui_info(self, proto, client_uuids, *args)
+                mixin_info = c.get_ui_info(self, proto, **kwargs)
                 log("%s.get_ui_info(%s, ..)=%r", c, proto, Ellipsizer(mixin_info))
                 merge_dicts(info, mixin_info)
         return info
 
-    def get_info(self, proto=None, client_uuids=None) -> dict[str, Any]:
-        log("ServerBase.get_info%s", (proto, client_uuids))
+    def get_threaded_info(self, proto, **kwargs) -> dict[str, Any]:
+        log("ServerBase.get_threaded_info%s", (proto, kwargs))
         start = monotonic()
+        info: dict[str, Any] = {}
+        subsystems = kwargs.get("subsystems", ())
+
+        def up(prefix, d) -> None:
+            merge_dicts(info, {prefix: d})
+
+        for c in SERVER_BASES:
+            if subsystems and subsystem_name(c) not in subsystems:
+                continue
+            with log.trap_error(f"Error collecting information from {c}"):
+                cstart = monotonic()
+                mixin_info = c.get_info(self, proto)
+                log("%s.get_info(%s)=%r", c, proto, Ellipsizer(mixin_info))
+                merge_dicts(info, mixin_info)
+                cend = monotonic()
+                log("%s.get_info(%s) took %ims", c, proto, int(1000 * (cend - cstart)))
+
+        if not subsystems or "features" in subsystems:
+            up("features", self.get_features_info())
+        if not subsystems or "network" in subsystems:
+            up("network", {
+                "sharing": self.sharing is not False,
+                "sharing-toggle": self.sharing is None,
+                "lock": self.lock is not False,
+                "lock-toggle": self.lock is None,
+            })
+        info["subsystems"] = self.get_subsystems()
+
+        sources = tuple(self._server_sources.values())
+        client_uuids = kwargs.get("client_uuids", ())
         if client_uuids:
-            sources = tuple(ss for ss in self._server_sources.values() if ss.uuid in client_uuids)
-        else:
-            sources = tuple(self._server_sources.values())
-        log("info-request: sources=%s", sources)
-        info = self.do_get_info(proto, sources)
+            sources = tuple(ss for ss in sources if ss.uuid in client_uuids)
+        log("threaded info sources(%i)=%s", client_uuids, sources)
+        info.update(self.get_sources_info(proto, sources))
         log("ServerBase.get_info took %.1fms", 1000.0 * (monotonic() - start))
         return info
 
@@ -501,35 +507,17 @@ class ServerBase(ServerBaseClass):
         i.update(self.get_server_features())
         return i
 
-    def do_get_info(self, proto, server_sources=()) -> dict[str, Any]:
-        log("ServerBase.do_get_info%s", (proto, server_sources))
+    def get_subsystems(self) -> list[str]:
+        return super().get_subsystems() + [subsystem_name(c) for c in SERVER_BASES]
+
+    def get_sources_info(self, proto, server_sources=()) -> dict[str, Any]:
+        log("ServerBase.get_source_info%s", (proto, server_sources))
         start = monotonic()
         info: dict[str, Any] = {}
 
         def up(prefix, d) -> None:
             merge_dicts(info, {prefix: d})
 
-        subsystems = []
-        for c in SERVER_BASES:
-            subsystems.append(c.__name__.replace("Server", ""))
-            with log.trap_error(f"Error collecting information from {c}"):
-                cstart = monotonic()
-                mixin_info = c.get_info(self, proto)
-                log("%s.get_info(%s)=%r", c, proto, Ellipsizer(mixin_info))
-                merge_dicts(info, mixin_info)
-                cend = monotonic()
-                log("%s.get_info(%s) took %ims", c, proto, int(1000 * (cend - cstart)))
-        up("subsystems", subsystems)
-
-        up("features", self.get_features_info())
-        up("network", {
-            "sharing": self.sharing is not False,
-            "sharing-toggle": self.sharing is None,
-            "lock": self.lock is not False,
-            "lock-toggle": self.lock is None,
-        })
-
-        # other clients:
         info["clients"] = {
             "": sum(1 for p in self._server_sources if p != proto),
             "unauthenticated": sum(1 for p in self._potential_protocols
@@ -550,10 +538,10 @@ class ServerBase(ServerBaseClass):
                 sinfo["ui-driver"] = self.ui_driver == ss.uuid
                 cinfo[i] = sinfo
             up("client", cinfo)
-        log("ServerBase.do_get_info took %ims", (monotonic() - start) * 1000)
+        log("ServerBase.get_source_info took %ims", (monotonic() - start) * 1000)
         return info
 
-    def _process_server_settings(self, proto, packet: PacketType) -> None:
+    def _process_server_settings(self, proto, packet: Packet) -> None:
         # only used by x11 servers
         pass
 
@@ -579,7 +567,7 @@ class ServerBase(ServerBaseClass):
                 if not k.startswith("encoding"):
                     ncp[k] = v
             if ncp:
-                log("set_client_properties updating window %s of source %s with %s", wid, ss.uuid, ncp)
+                log("set_client_properties updating window %#x of source %s with %s", wid, ss.uuid, ncp)
                 client_properties = self.client_properties.setdefault(wid, {}).setdefault(ss.uuid, {})
                 client_properties.update(ncp)
 
@@ -590,12 +578,12 @@ class ServerBase(ServerBaseClass):
         for ss in tuple(self._server_sources.values()):
             ss.send_setting_change(setting, value)
 
-    def _process_sharing_toggle(self, proto, packet: PacketType) -> None:
+    def _process_sharing_toggle(self, proto, packet: Packet) -> None:
         assert self.sharing is None
         ss = self.get_server_source(proto)
         if not ss:
             return
-        sharing = bool(packet[1])
+        sharing = packet.get_bool(1)
         ss.share = sharing
         if not sharing:
             # disconnect other users:
@@ -604,11 +592,11 @@ class ServerBase(ServerBaseClass):
                     self.disconnect_client(p, ConnectionMessage.DETACH_REQUEST,
                                            f"client {ss.counter} no longer wishes to share the session")
 
-    def _process_lock_toggle(self, proto, packet: PacketType) -> None:
+    def _process_lock_toggle(self, proto, packet: Packet) -> None:
         assert self.lock is None
         ss = self.get_server_source(proto)
         if ss:
-            ss.lock = bool(packet[1])
+            ss.lock = packet.get_bool(1)
             log("lock set to %s for client %i", ss.lock, ss.counter)
 
     ######################################################################
@@ -635,7 +623,8 @@ class ServerBase(ServerBaseClass):
         source = self._server_sources.pop(protocol, None)
         if source:
             self.cleanup_source(source)
-            add_work_item(self.mdns_update)
+            mdns_update = getattr(self, "mdns_update", noop)
+            add_work_item(mdns_update)
         for c in SERVER_BASES:
             c.cleanup_protocol(self, protocol)
         return source
@@ -645,6 +634,10 @@ class ServerBase(ServerBaseClass):
         if FULL_INFO > 0:
             ptype = getattr(source, "client_type", "") or "xpra"
         self.server_event("connection-lost", source.uuid)
+        for bc in SERVER_BASES:
+            with log.trap_error("Error removing client %s from %s", source, bc):
+                bc.remove_client(self, source)
+
         remaining_sources = tuple(self._server_sources.values())
         if self.ui_driver == source.uuid:
             if len(remaining_sources) == 1:
@@ -661,7 +654,6 @@ class ServerBase(ServerBaseClass):
     def last_client_exited(self) -> None:
         # must run from the UI thread (modifies focus and keys)
         netlog("last_client_exited() exit_with_client=%s", self.exit_with_client)
-        self.reset_server_timeout(True)
         for c in SERVER_BASES:
             if c != ServerCore:
                 try:
@@ -688,7 +680,7 @@ class ServerBase(ServerBaseClass):
         return list(self._potential_protocols) + list(self._server_sources.keys())
 
     def is_timedout(self, protocol) -> bool:
-        v = ServerCore.is_timedout(self, protocol) and protocol not in self._server_sources
+        v = super().is_timedout(protocol) and protocol not in self._server_sources
         netlog("is_timedout(%s)=%s", protocol, v)
         return v
 
@@ -704,20 +696,21 @@ class ServerBase(ServerBaseClass):
         # no need for main thread:
         self.add_packets("sharing-toggle", "lock-toggle")
         self.add_packet_handler("set_deflate", noop)  # removed in v6
-        # attributes / settings:
-        self.add_packets("server-settings", "info-request", main_thread=True)
+        if BACKWARDS_COMPATIBLE:
+            # now moved to XSettingsServer
+            self.add_packets("server-settings", main_thread=True)
         self.add_packets("shutdown-server", "exit-server")
 
     # override so we can set the 'authenticated' flag:
-    def process_packet(self, proto, packet) -> None:
+    def process_packet(self, proto, packet: Packet) -> None:
         authenticated = bool(self.get_server_source(proto))
         return super().dispatch_packet(proto, packet, authenticated)
 
-    def handle_invalid_packet(self, proto, packet) -> None:
+    def handle_invalid_packet(self, proto, packet: Packet) -> None:
         ss = self.get_server_source(proto)
         if not self._closing and not proto.is_closed() and (ss is None or not ss.is_closed()):
             netlog("invalid packet: %s", packet)
-            packet_type = str(packet[0])
+            packet_type = packet.get_type()
             netlog.error(f"Error: unknown or invalid packet type {packet_type!r}")
             netlog.error(f" received from {proto}")
         if not ss:
