@@ -1,5 +1,5 @@
 # This file is part of Xpra.
-# Copyright (C) 2012-2024 Antoine Martin <antoine@xpra.org>
+# Copyright (C) 2012 Antoine Martin <antoine@xpra.org>
 # Xpra is released under the terms of the GNU GPL v2, or, at your option, any
 # later version. See the file COPYING for details.
 
@@ -9,15 +9,14 @@ from collections.abc import Sequence, Callable
 
 from xpra.os_util import gi_import
 from xpra.server.window import batch_config
-from xpra.server.shadow.root_window_model import RootWindowModel
+from xpra.server.base import ServerBase
 from xpra.scripts.config import InitExit
-from xpra.platform.gui import get_native_notifier_classes, get_wm_name
+from xpra.platform.gui import get_wm_name
 from xpra.platform.paths import get_icon_dir
 from xpra.server import features
 from xpra.exit_codes import ExitCode
-from xpra.net.common import PacketType
+from xpra.net.common import Packet
 from xpra.exit_codes import ExitValue
-from xpra.util.system import is_Wayland
 from xpra.util.env import envint, envbool
 from xpra.util.str_fn import csv
 from xpra.common import NotificationID, ConnectionMessage
@@ -27,7 +26,7 @@ GLib = gi_import("GLib")
 
 log = Logger("shadow")
 notifylog = Logger("notify")
-mouselog = Logger("mouse")
+pointerlog = Logger("pointer")
 cursorlog = Logger("cursor")
 
 NATIVE_NOTIFIER = envbool("XPRA_NATIVE_NOTIFIER", True)
@@ -36,11 +35,6 @@ CURSORS = envbool("XPRA_CURSORS", True)
 SAVE_CURSORS = envbool("XPRA_SAVE_CURSORS", False)
 NOTIFY_STARTUP = envbool("XPRA_SHADOW_NOTIFY_STARTUP", True)
 
-SHADOWSERVER_BASE_CLASS: type = object
-if features.rfb:
-    from xpra.server.rfb.server import RFBServer
-    SHADOWSERVER_BASE_CLASS = RFBServer
-
 
 def try_setup_capture(backends: dict[str, Callable], backend: str, *args):
     backend = backend.lower()
@@ -49,33 +43,33 @@ def try_setup_capture(backends: dict[str, Callable], backend: str, *args):
             raise InitExit(ExitCode.UNSUPPORTED,
                            f"invalid capture backend {backend!r}, should be one of: %s" % csv(backends.keys()))
         backends = {backend: backends[backend]}
-    log(f"setup_capture() will try {backends.keys()}")
+    log(f"setup_capture() will try {csv(backends.keys())}")
     for backend, setup_fn in backends.items():
         try:
             log(f"{backend!r}: {setup_fn}{args}")
             capture = setup_fn(*args)
             if not capture:
-                log(f"backend {backend!r} is unable to capture")
+                log(f"backend {backend!r} is unable to capture using {setup_fn!r}({csv(args)})")
                 continue
             return capture
         except ImportError as e:
-            log(f"backend {backend!r} is not installed: {e}")
+            log.info(f"shadow backend {backend!r} is not installed: {e}")
         except Exception as e:
             log(f"{setup_fn}{args}", exc_info=True)
             log.warn(f"Warning: {backend!r} failed to setup screen capture:")
             log.warn(" %s", e)
-    raise InitExit(ExitCode.UNSUPPORTED, f"failed to setup screen capture backend {backend!r}")
+    raise InitExit(ExitCode.UNSUPPORTED, f"failed to setup screen capture backends: {csv(backends)}")
 
 
-class ShadowServerBase(SHADOWSERVER_BASE_CLASS):
+class ShadowServerBase(ServerBase):
+    SIGNALS = ServerBase.__signals__
     # 20 fps unless the client specifies more:
     DEFAULT_REFRESH_RATE: int = 20
 
-    def __init__(self, root_window, capture=None):
+    def __init__(self, capture=None):
         # noinspection PyArgumentList
-        super().__init__()
+        ServerBase.__init__(self)
         self.capture = capture
-        self.root = root_window
         self.window_matches: list[str] = []
         self.mapped = []
         self.pulseaudio: bool = False
@@ -88,27 +82,29 @@ class ShadowServerBase(SHADOWSERVER_BASE_CLASS):
         self.pointer_poll_timer = 0
         self.last_cursor_data = None
         self.session_name = "shadow"
+        self.session_type = "shadow"
         self.keyboard_config = None
         batch_config.ALWAYS = True  # always batch
 
     def init(self, opts) -> None:
-        if SHADOWSERVER_BASE_CLASS is not object:
-            # RFBServer:
-            SHADOWSERVER_BASE_CLASS.init(self, opts)
+        super().init(opts)
         self.notifications = bool(opts.notifications)
         if self.notifications:
             self.make_notifier()
         log("init(..) session_name=%s", opts.session_name)
         if opts.session_name:
             self.session_name = opts.session_name
-        else:
-            self.guess_session_name()
 
     def run(self) -> ExitValue:
         if NOTIFY_STARTUP:
             GLib.timeout_add(1000, self.notify_startup_complete)
         return super().run()
 
+    def setup(self) -> None:
+        if not self.session_name:
+            GLib.idle_add(self.guess_session_name)
+        super().setup()
+
     def cleanup(self) -> None:
         for wid in self.mapped:
             self.stop_refresh(wid)
@@ -121,51 +117,31 @@ class ShadowServerBase(SHADOWSERVER_BASE_CLASS):
             self.capture = None
             capture.clean()
 
-    def guess_session_name(self, procs=None) -> None:
+    def guess_session_name(self, procs=()) -> None:
         log("guess_session_name(%s)", procs)
         self.session_name = get_wm_name()  # pylint: disable=assignment-from-none
         log("get_wm_name()=%s", self.session_name)
 
-    def get_server_mode(self) -> str:
-        return "shadow"
-
-    def print_screen_info(self) -> None:
-        if not features.display or not self.root:
-            return
-        w, h = self.root.get_geometry()[2:4]
-        dinfo = ""
-        if is_Wayland():
-            wdisplay = os.environ.get("WAYLAND_DISPLAY", "").replace("wayland-", "")
-            if wdisplay:
-                dinfo = f"Wayland display {wdisplay}"
-        else:
-            display = os.environ.get("DISPLAY")
-            if display:
-                dinfo = f"X11 display {display}"
-        self.do_print_screen_info(dinfo, w, h)
-
-    def do_print_screen_info(self, display: str, w: int, h: int) -> None:
-        if display or w or h:
-            if display:
-                msg = f" on display {display!r}"
-                if w and h:
-                    msg += f" of size {w}x{h}"
-            else:
-                msg = f" on display of size {w}x{h}"
-            log.info(msg)
+    def get_display_description(self) -> None:
+        descr = super().get_display_description()
         if self.window_matches:
-            return
+            return descr
         try:
             count = len(self._id_to_window)
         except AttributeError as e:
             log(f"no screen info: {e}")
-            return
+            return descr
         if count > 1:
-            log.info(f" with {count} monitors:")
+            descr += f"\n with {count} monitors:"
             for window in self._id_to_window.values():
                 title = window.get_property("title")
                 x, y, w, h = window.geometry
-                log.info("  %-16s %4ix%-4i at %4i,%-4i", title, w, h, x, y)
+                descr += "\n  %-16s %4ix%-4i at %4i,%-4i" % (title, w, h, x, y)
+        return descr
+
+    @staticmethod
+    def set_desktop_geometry(w: int, h: int) -> None:
+        pass
 
     def apply_refresh_rate(self, ss) -> None:
         rrate = super().apply_refresh_rate(ss)
@@ -173,14 +149,17 @@ class ShadowServerBase(SHADOWSERVER_BASE_CLASS):
             # adjust refresh delay to try to match:
             self.set_refresh_delay(max(10, 1000 // rrate))
 
-    def make_hello(self, _source) -> dict[str, Any]:
-        return {"shadow": True}
+    def make_hello(self, source) -> dict[str, Any]:
+        hello = super().make_hello(source)
+        hello["shadow"] = True
+        return hello
 
-    def get_info(self, _proto=None, *args) -> dict[str, Any]:
-        info = {
+    def get_threaded_info(self, proto, **kwargs) -> dict[str, Any]:
+        info = super().get_threaded_info(proto, **kwargs)
+        info.update({
             "sharing": self.sharing is not False,
             "refresh-delay": self.refresh_delay,
-        }
+        })
         if self.pointer_last_position:
             info["pointer-last-position"] = self.pointer_last_position
         return info
@@ -190,14 +169,14 @@ class ShadowServerBase(SHADOWSERVER_BASE_CLASS):
         return 0, 0
 
     def _keys_changed(self) -> None:
-        from xpra.server.mixins.input import InputServer
-        if isinstance(self, InputServer):
-            InputServer._keys_changed(self)
+        from xpra.server.subsystem.keyboard import KeyboardServer
+        if isinstance(self, KeyboardServer):
+            KeyboardServer._keys_changed(self)
             from xpra.platform.keyboard import Keyboard
             log.info("the keymap has been changed: %s", Keyboard().get_layout_spec()[0])
 
     ############################################################################
-    # notifications
+    # notification
     def cleanup_notifier(self) -> None:
         n = self.notifier
         if n:
@@ -221,13 +200,14 @@ class ShadowServerBase(SHADOWSERVER_BASE_CLASS):
             except Exception:
                 notifylog("failed to instantiate %s", nclass, exc_info=True)
 
-    def get_notifier_classes(self) -> list[type]:
+    def get_notifier_classes(self) -> list[Callable]:
         # subclasses will generally add their toolkit specific variants
         # by overriding this method
         # use the native ones first:
         if not NATIVE_NOTIFIER:
             return []
-        return get_native_notifier_classes()
+        from xpra.platform.notification import get_backends
+        return get_backends()
 
     def notify_new_user(self, ss) -> None:
         # overridden here so that we can show the notification
@@ -238,10 +218,10 @@ class ShadowServerBase(SHADOWSERVER_BASE_CLASS):
             nid = NotificationID.NEW_USER
             title = "User '%s' connected to the session" % (ss.name or ss.username or ss.uuid)
             body = "\n".join(ss.get_connect_info())
-            actions: list = []
+            actions: Sequence[str] = ()
             hints: dict[str, Any] = {}
             icon_filename = os.path.join(get_icon_dir(), "user.png")
-            from xpra.notifications.common import parse_image_path
+            from xpra.notification.common import parse_image_path
             icon = parse_image_path(icon_filename)
             self.notifier.show_notify("", tray, nid, "Xpra", 0, "", title, body, actions, hints, 10 * 1000, icon)
 
@@ -257,10 +237,10 @@ class ShadowServerBase(SHADOWSERVER_BASE_CLASS):
         notifylog("do_notify_startup%s", (title, body, replaces_nid))
         if self.notifier:
             tray = self.get_notification_tray()  # pylint: disable=assignment-from-none
-            actions = []
+            actions: Sequence[str] = ()
             hints = {}
             icon_filename = os.path.join(get_icon_dir(), "server-connected.png")
-            from xpra.notifications.common import parse_image_path
+            from xpra.notification.common import parse_image_path
             icon = parse_image_path(icon_filename)
             self.notifier.show_notify("", tray, NotificationID.STARTUP, "Xpra", replaces_nid, "",
                                       title, body, actions, hints, 10 * 1000, icon)
@@ -269,7 +249,7 @@ class ShadowServerBase(SHADOWSERVER_BASE_CLASS):
     # refresh
 
     def start_refresh(self, wid: int) -> None:
-        log("start_refresh(%i) mapped=%s, timer=%s", wid, self.mapped, self.refresh_timer)
+        log("start_refresh(%#x) mapped=%s, timer=%s", wid, self.mapped, self.refresh_timer)
         if wid not in self.mapped:
             self.mapped.append(wid)
         self.start_refresh_timer()
@@ -288,10 +268,10 @@ class ShadowServerBase(SHADOWSERVER_BASE_CLASS):
                 self.start_refresh(wid)
 
     def stop_refresh(self, wid: int) -> None:
-        log("stop_refresh(%i) mapped=%s", wid, self.mapped)
+        log("stop_refresh(%#x) mapped=%s", wid, self.mapped)
         try:
             self.mapped.remove(wid)
-        except KeyError:
+        except ValueError:
             pass
         if not self.mapped:
             self.no_windows()
@@ -313,15 +293,15 @@ class ShadowServerBase(SHADOWSERVER_BASE_CLASS):
     ############################################################################
     # pointer polling
 
-    def get_pointer_position(self):
+    def get_pointer_position(self) -> tuple[int, int]:
         raise NotImplementedError()
 
     def start_poll_pointer(self) -> None:
-        log("start_poll_pointer() pointer_poll_timer=%s, input_devices=%s, POLL_POINTER=%s",
-            self.pointer_poll_timer, features.input_devices, POLL_POINTER)
+        log("start_poll_pointer() pointer_poll_timer=%s, pointer=%s, POLL_POINTER=%s",
+            self.pointer_poll_timer, features.pointer, POLL_POINTER)
         if self.pointer_poll_timer:
             self.cancel_poll_pointer()
-        if features.input_devices and POLL_POINTER > 0:
+        if features.pointer and POLL_POINTER > 0:
             self.pointer_poll_timer = GLib.timeout_add(POLL_POINTER, self.poll_pointer)
 
     def cancel_poll_pointer(self) -> None:
@@ -340,7 +320,7 @@ class ShadowServerBase(SHADOWSERVER_BASE_CLASS):
     def poll_pointer_position(self) -> None:
         x, y = self.get_pointer_position()
         if self.pointer_last_position == (x, y):
-            mouselog("poll_pointer_position() unchanged position=%s", (x, y))
+            pointerlog("poll_pointer_position() unchanged position=%s", (x, y))
             return
         self.pointer_last_position = (x, y)
         rwm = None
@@ -355,9 +335,9 @@ class ShadowServerBase(SHADOWSERVER_BASE_CLASS):
                 ry = y - wy
                 break
         if not rwm:
-            mouselog("poll_pointer_position() model not found for position=%s", (x, y))
+            pointerlog("poll_pointer_position() model not found for position=%s", (x, y))
             return
-        mouselog("poll_pointer_position() wid=%i, position=%s, relative=%s", wid, (x, y), (rx, ry))
+        pointerlog("poll_pointer_position() wid=%#x, position=%s, relative=%s", wid, (x, y), (rx, ry))
         for ss in self._server_sources.values():
             um = getattr(ss, "update_mouse", None)
             if um:
@@ -396,7 +376,8 @@ class ShadowServerBase(SHADOWSERVER_BASE_CLASS):
                     img = Image.frombuffer("RGBA", (w, h), pixels, "raw", "BGRA", 0, 1)
                     img.save("cursor-%#x.png" % serial, format="PNG")
             for ss in self.window_sources():
-                ss.send_cursor()
+                if hasattr(ss, "send_cursor"):
+                    ss.send_cursor()
 
     def do_get_cursor_data(self):
         # this method is overridden in subclasses with platform specific code
@@ -428,17 +409,16 @@ class ShadowServerBase(SHADOWSERVER_BASE_CLASS):
         except Exception:
             log.info(" unknown client desktop size")
         self.apply_refresh_rate(ss)
-        return self.get_root_window_size()
+        return self.get_display_size()
 
-    def _process_desktop_size(self, proto, packet: PacketType) -> None:
+    def _process_desktop_size(self, proto, packet: Packet) -> None:
         # just record the screen size info in the source
         ss = self.get_server_source(proto)
         if ss and len(packet) >= 4:
             ss.set_screen_sizes(packet[3])
 
-    def set_keyboard_repeat(self, key_repeat) -> None:
+    def set_keyboard_repeat(self, *_args) -> None:
         """ don't override the existing desktop """
-        pass  # pylint: disable=unnecessary-pass
 
     def set_keymap(self, server_source, force=False) -> None:
         log("set_keymap%s", (server_source, force))
@@ -446,16 +426,17 @@ class ShadowServerBase(SHADOWSERVER_BASE_CLASS):
         self.keyboard_config = server_source.set_default_keymap()
 
     def load_existing_windows(self) -> None:
-        self.min_mmap_size = 1024 * 1024 * 4 * 2
-        for i, model in enumerate(self.makeRootWindowModels()):
+        self.mmap_min_size = 1024 * 1024 * 4 * 2
+        for i, model in enumerate(self.make_capture_window_models()):
             log(f"load_existing_windows() root window model {i} : {model}")
             self._add_new_window(model)
             # at least big enough for 2 frames of BGRX pixel data:
             w, h = model.get_dimensions()
-            self.min_mmap_size = max(self.min_mmap_size, w * h * 4 * 2)
+            self.mmap_min_size = max(self.mmap_min_size, w * h * 4 * 2)
 
-    def makeRootWindowModels(self) -> list:
-        return [RootWindowModel(self.root)]
+    def make_capture_window_models(self) -> list:
+        from xpra.server.shadow.root_window_model import CaptureWindowModel
+        return [CaptureWindowModel()]
 
     def send_initial_windows(self, ss, sharing: bool = False) -> None:
         log("send_initial_windows(%s, %s) will send: %s", ss, sharing, self._id_to_window)
@@ -483,20 +464,24 @@ class ShadowServerBase(SHADOWSERVER_BASE_CLASS):
     def process_window_common(self, wid: int):
         return self._id_to_window.get(wid)
 
-    def _process_map_window(self, proto, packet: PacketType) -> None:
-        wid, x, y, width, height = packet[1:6]
+    def _process_map_window(self, proto, packet: Packet) -> None:
+        wid = packet.get_wid()
+        x = packet.get_i16(2)
+        y = packet.get_i16(3)
+        w = packet.get_i16(4)
+        h = packet.get_i16(5)
         window = self.process_window_common(wid)
         if not window:
             # already gone
             return
-        self._window_mapped_at(proto, wid, window, (x, y, width, height))
-        self.refresh_window_area(window, 0, 0, width, height)
+        self._window_mapped_at(proto, wid, window, (x, y, w, h))
+        self.refresh_window_area(window, 0, 0, w, h)
         if len(packet) >= 7:
             self._set_client_properties(proto, wid, window, packet[6])
         self.start_refresh(wid)
 
-    def _process_unmap_window(self, proto, packet: PacketType) -> None:
-        wid = packet[1]
+    def _process_unmap_window(self, proto, packet: Packet) -> None:
+        wid = packet.get_wid()
         window = self.process_window_common(wid)
         if not window:
             # already gone
@@ -507,8 +492,12 @@ class ShadowServerBase(SHADOWSERVER_BASE_CLASS):
         if len(self._server_sources) <= 1 and len(self._id_to_window) <= 1:
             self.stop_refresh(wid)
 
-    def _process_configure_window(self, proto, packet: PacketType) -> None:
-        wid, x, y, w, h = packet[1:6]
+    def _process_configure_window(self, proto, packet: Packet) -> None:
+        wid = packet.get_wid()
+        x = packet.get_i16(2)
+        y = packet.get_i16(3)
+        w = packet.get_i16(4)
+        h = packet.get_i16(5)
         window = self.process_window_common(wid)
         if not window:
             # already gone
@@ -518,8 +507,8 @@ class ShadowServerBase(SHADOWSERVER_BASE_CLASS):
         if len(packet) >= 7:
             self._set_client_properties(proto, wid, window, packet[6])
 
-    def _process_close_window(self, proto, packet: PacketType) -> None:
-        wid = packet[1]
+    def _process_close_window(self, proto, packet: Packet) -> None:
+        wid = packet.get_wid()
         window = self.process_window_common(wid)
         if not window:
             # already gone
