@@ -1,6 +1,6 @@
 # This file is part of Xpra.
 # Copyright (C) 2011 Serviware (Arthur Huillet, <ahuillet@serviware.com>)
-# Copyright (C) 2010-2024 Antoine Martin <antoine@xpra.org>
+# Copyright (C) 2010 Antoine Martin <antoine@xpra.org>
 # Copyright (C) 2008 Nathaniel Smith <njs@pobox.com>
 # Xpra is released under the terms of the GNU GPL v2, or, at your option, any
 # later version. See the file COPYING for details.
@@ -11,9 +11,8 @@ from typing import Any, Deque
 from collections.abc import Sequence
 
 from collections import deque
-from xpra.common import Self
+from xpra.common import Self, MIN_VREFRESH, MAX_VREFRESH
 from xpra.util.stats import get_list_stats
-from xpra.util.env import envint
 from xpra.log import Logger
 
 # how many historical records to keep
@@ -21,9 +20,6 @@ from xpra.log import Logger
 # (cannot be lower than DamageBatchConfig.MAX_EVENTS)
 NRECS: int = 100
 
-MIN_VREFRESH: int = envint("XPRA_MIN_VREFRESH", 1)
-MAX_VREFRESH: int = envint("XPRA_MAX_VREFRESH", 250)
-
 log = Logger("damage")
 
 
@@ -152,4 +148,4 @@ class DamageBatchConfig:
         return c
 
     def __repr__(self):
-        return f"DamageBatchConfig({self.wid})"
+        return f"DamageBatchConfig({self.wid:#x})"
