@@ -10,12 +10,13 @@ from collections.abc import Callable, Sequence
 from xpra.os_util import gi_import
 from xpra.util.env import first_time
 from xpra.common import WORKSPACE_UNSET, WORKSPACE_ALL
-from xpra.gtk.error import xsync, xlog
+from xpra.x11.common import _NET_WM_STATE_ADD, _NET_WM_STATE_REMOVE, _NET_WM_STATE_TOGGLE, STATE_STRING
+from xpra.x11.error import xsync, xlog
 from xpra.x11.models.core import CoreX11WindowModel, Above, RESTACKING_STR
-from xpra.x11.bindings.window import X11WindowBindings, constants
+from xpra.x11.bindings.core import constants
+from xpra.x11.bindings.window import X11WindowBindings
 from xpra.server.window.content_guesser import guess_content_type, get_content_type_properties, GUESS_CONTENT
-from xpra.server.background_worker import add_work_item
-from xpra.x11.gtk.bindings import get_pywindow, get_pyatom
+from xpra.util.background_worker import add_work_item
 from xpra.log import Logger
 
 log = Logger("x11", "window")
@@ -26,15 +27,12 @@ GObject = gi_import("GObject")
 
 X11Window = X11WindowBindings()
 
-# _NET_WM_STATE:
-_NET_WM_STATE_REMOVE = 0
-_NET_WM_STATE_ADD = 1
-_NET_WM_STATE_TOGGLE = 2
-STATE_STRING: dict[int, str] = {
-    _NET_WM_STATE_REMOVE: "REMOVE",
-    _NET_WM_STATE_ADD: "ADD",
-    _NET_WM_STATE_TOGGLE: "TOGGLE",
-}
+
+def get_pyatom(value: int):
+    with xsync:
+        return X11Window.get_atom_name(value)
+
+
 WithdrawnState: Final[int] = constants["WithdrawnState"]
 IconicState: Final[int] = constants["IconicState"]
 NormalState: Final[int] = constants["NormalState"]
@@ -78,8 +76,9 @@ class BaseWindowModel(CoreX11WindowModel):
         ),
         # from WM_HINTS.window_group
         "group-leader": (
-            GObject.TYPE_PYOBJECT,
-            "Window group leader as a pair: (xid, gdk window)", "",
+            GObject.TYPE_INT,
+            "Window group leader", "",
+            -1, 65535, -1,
             GObject.ParamFlags.READABLE,
         ),
         # from WM_HINTS.urgency or _NET_WM_STATE
@@ -106,7 +105,7 @@ class BaseWindowModel(CoreX11WindowModel):
         # from _NET_WM_FULLSCREEN_MONITORS
         "fullscreen-monitors": (
             GObject.TYPE_PYOBJECT,
-            "list of 4 monitor indices indicating the top, bottom, left, and right edges"
+            "tuple of 4 monitor indices indicating the top, bottom, left, and right edges"
             " of the window when the fullscreen state is enabled", "",
             GObject.ParamFlags.READABLE,
         ),
@@ -295,7 +294,7 @@ class BaseWindowModel(CoreX11WindowModel):
     def _read_initial_X11_properties(self) -> None:
         metalog("%s.read_initial_X11_properties()", self._MODELTYPE)
         wm_state = tuple(self._read_wm_state())
-        self._updateprop("state", frozenset(wm_state))
+        self._updateprop("state", wm_state)
         self._gproperties["iconic"] = "_NET_WM_STATE_HIDDEN" in wm_state
         super()._read_initial_X11_properties()
         self._update_content_type()
@@ -342,12 +341,12 @@ class BaseWindowModel(CoreX11WindowModel):
             # a window should not be focused and hidden at the same time:
             state = set(state)
             state.remove("_NET_WM_STATE_HIDDEN")
-            state = frozenset(state)
+            state = tuple(state)
         metalog("sync_state: setting _NET_WM_STATE=%s on %#x", state, self.xid)
         with xlog:
             self.prop_set("_NET_WM_STATE", ["atom"], state)
 
-    def set_wm_state(self, state: int):
+    def set_wm_state(self, state: int) -> None:
         metalog("set_wm_state(%s)", state)
         with xlog:
             self.prop_set("WM_STATE", "state", state)
@@ -371,11 +370,10 @@ class BaseWindowModel(CoreX11WindowModel):
     #########################################
 
     def _handle_transient_for_change(self) -> None:
-        transient_for = self.prop_get("WM_TRANSIENT_FOR", "window")
-        metalog("WM_TRANSIENT_FOR=%s", transient_for)
+        transient_for = self.prop_get("WM_TRANSIENT_FOR", "window") or 0
+        metalog("WM_TRANSIENT_FOR=%#x", transient_for)
         # `transient-for` may be None
-        window = get_pywindow(transient_for) if transient_for else None
-        self._updateprop("transient-for", window)
+        self._updateprop("transient-for", transient_for)
 
     def _handle_window_type_change(self) -> None:
         window_types = self.prop_get("_NET_WM_WINDOW_TYPE", ["atom"])
@@ -399,7 +397,7 @@ class BaseWindowModel(CoreX11WindowModel):
     def _handle_fullscreen_monitors_change(self) -> None:
         fsm = self.prop_get("_NET_WM_FULLSCREEN_MONITORS", ["u32"], True)
         metalog("_NET_WM_FULLSCREEN_MONITORS=%s", fsm)
-        self._updateprop("fullscreen-monitors", fsm)
+        self._updateprop("fullscreen-monitors", tuple(fsm or ()))
 
     def _handle_bypass_compositor_change(self) -> None:
         bypass = self.prop_get("_NET_WM_BYPASS_COMPOSITOR", "u32", True) or 0
@@ -412,8 +410,8 @@ class BaseWindowModel(CoreX11WindowModel):
         if strut is None:
             strut = self.prop_get("_NET_WM_STRUT", "strut")
             metalog("_NET_WM_STRUT=%s", strut)
-        # Might be None:
-        self._updateprop("strut", strut)
+        value = {} if not strut else strut.todict()
+        self._updateprop("strut", value)
 
     def _handle_opacity_change(self) -> None:
         opacity = self.prop_get("_NET_WM_WINDOW_OPACITY", "u32", True) or -1
@@ -427,13 +425,9 @@ class BaseWindowModel(CoreX11WindowModel):
         if wm_hints is None:
             return
         # GdkWindow or None
-        group_leader = None
+        group_leader = 0
         if "window_group" in wm_hints:
-            xid = wm_hints.get("window_group")
-            if xid:
-                group_leader = get_pywindow(xid)
-                if not group_leader:
-                    log.error(f"Error group leader window {xid:x} not found")
+            group_leader = wm_hints.get("window_group", 0)
         self._updateprop("group-leader", group_leader)
         self._updateprop("attention-requested", wm_hints.get("urgency", False))
         _input = wm_hints.get("input")
@@ -555,7 +549,7 @@ class BaseWindowModel(CoreX11WindowModel):
                 add, remove, was, added, removed, curr)
         if added or removed:
             # note: _sync_state will update _NET_WM_STATE here:
-            self._internal_set_property("state", frozenset(curr))
+            self._internal_set_property("state", tuple(curr))
             self._state_notify(added | removed)
 
     def _state_notify(self, state_names: set[str]) -> None:
@@ -699,10 +693,13 @@ class BaseWindowModel(CoreX11WindowModel):
             workspace = int(event.data[0])
             # query the workspace count on the root window
             # since we cannot access Wm from here…
-            ndesktops = int(self.root_prop_get("_NET_NUMBER_OF_DESKTOPS", "u32") or 0)
+            from xpra.x11.common import get_number_of_desktops
+            ndesktops = get_number_of_desktops()
             workspacelog("received _NET_WM_DESKTOP: workspace=%s, number of desktops=%s",
                          workspacestr(workspace), ndesktops)
-            if ndesktops > 0 and (workspace in (WORKSPACE_UNSET, WORKSPACE_ALL) or 0 <= workspace < ndesktops):
+            if ndesktops == 0:
+                self.move_to_workspace(0)
+            elif ndesktops > 0 and (workspace in (WORKSPACE_UNSET, WORKSPACE_ALL) or 0 <= workspace < ndesktops):
                 self.move_to_workspace(workspace)
             else:
                 workspacelog.warn("invalid _NET_WM_DESKTOP request: workspace=%s, number of desktops=%s",
